import numpy as np
from typing import Callable, Any
from scipy.optimize import linear_sum_assignment

class HungarianScorer():
    @staticmethod
    def getBestCombo(listA:list, listB:list, scoringFunc:Callable, *args) -> list[tuple[Any, Any, float]]:
        bestCombo = HungarianScorer.getBestComboAsIndices(listA, listB, scoringFunc, *args)
        decodedCombo = []
        for i, j, score in bestCombo:
            tup = (listA[i], listB[j], score)
            decodedCombo.append(tup)
        return decodedCombo

    @staticmethod
    def getBestComboAsIndices(listA:list, listB:list, scoringFunc:Callable, *args) -> list[tuple[Any, Any, float]]:    
        # Ensure the same number of items for both lists, padding with dummy values if necessary
        if len(listA) != len(listB):
            if len(listA) < len(listB):
                listA += [None] * (len(listB) - len(listA))
            else:
                listB += [None] * (len(listA) - len(listB))

        # Score each matchup
        scores = np.zeros((len(listA), len(listB)))
        for i, itemA in enumerate(listA):
            for j, itemB in enumerate(listB):
                if (itemA is not None) and (itemB is not None):
                    scores[i, j] = scoringFunc(itemA, itemB, *args)
                else:
                    scores[i, j] = -1e9  # Assign a very low finite score to dummy pairings

        # Use the Hungarian algorithm to find the optimal assignments
        rowInd, colInd = linear_sum_assignment(-scores)

        # Format the best combination into a readable form
        bestCombination = []
        for i, j in zip(rowInd, colInd):
            if (listA[i] is not None) and (listB[j] is not None):
                matchupScore = scores[i, j]
                bestCombination.append((i, j, matchupScore))
        return bestCombination


import setuptools
import os
from _version import __version__ as version

with open("README.md", "r", encoding="utf-8") as fh:
    longDescription = fh.read()

requirements = ""
with open("HungarianScorer/requirements.txt", "r", encoding="utf-8") as fh:
    requirements = fh.read()

requirements = requirements.split("\n")

def listFolders(directory: str) -> list:
    """Creates a list of all the folders in a directory.

    Args:
        directory (str): the directory to search

    Returns:
        list: A list of all the folders in the directory
    """
    folders = []
    for item in os.listdir(directory):
        itemPath = os.path.join(directory, item)
        if os.path.isdir(itemPath) and item != "__pycache__":
            folders.append(itemPath)
    otherFolders = [listFolders(itemPath) for itemPath in folders]
    for folder in otherFolders:
        folders.extend(folder)
    return folders

folderPath = "HungarianScorer"
folders = listFolders(folderPath)
folders.append("HungarianScorer")
print(folders)

setuptools.setup(
    name='HungarianScorer',
    version=version,
    author='Johnny Weymouth',
    author_email='johnnycweymouth@gmail.com',
    description='This package makes finding optimal combinations using the hungarian algorithm dead simple.',
    long_description=longDescription,
    long_description_content_type="text/markdown",
    url='https://github.com/johnnyweymouth/HungarianScorer.git',
    project_urls = {
        "Bug Tracker": "https://github.com/johnnyweymouth/HungarianScorer/issues"
    },
    packages=folders,
    install_requires=requirements,
    package_data={"": ["*.json", "*.txt"]},
)